/************************************************************
* Synchronous neuro-controller for Aibo walking using URBI *
* by Ricardo A. Tellez r_tellez-at-ouroboros.org            *
*  22 - MAY - 2006   More info at www.ouroboros.org			*
************************************************************/

#include <iostream>
#include <time.h>
#include <fstream.h>
#include <unistd.h>

#include "Neurocontrolado.h"

#define NUM_NETS 24
#define NUM_JOINTS 12
#define NUM_SENSORS 12

// definitions of some useful functions and global variables
double JointLastValue[NUM_JOINTS];
void load_weights_from_file ();

using namespace std;

// global variables
Neuron * neuron;
Network * network;
std::vector< Network* > pNet;  // contains the vector of (pointers to) nets of the controller
std::vector<double> input(inputDimension);
std::vector<double> output[NUM_NETS];
int state;		// global variable that indicates if the neurocontroller is active or stopped
int blending_mode;		// global variable that indicates the present blending mode active
int headCounter = 0;
int backCounter = 0;

Neurocontrolador::Neurocontrolador()
{
	
	double tmp;
	
	//Inicialization of inputs and outputs vectors for the use in the neural nets
	for(int i=0;i < inputDimension;i++)
	{
		input[i]=0;
		output[i]=std::vector<double>(outputDimension);
		for (int j=0; j<outputDimension; j++)
			output[i][j] = 0;
	}
	
	// Opens files containing neural nets weights and creates the nets from that data
	load_weights_from_file();
	cout << "NEURAL NETWORKS CREATED\n"<< flush;

	// Client Urbi connects with server in robot
	Travolta = new USyncClient("192.168.10.100");  // Creates a new client for the control of the robot	
   	
	if (Travolta->error() != 0)  //  Validation of the client creation
    	exit(0);
	
	sampletime=96000; // in Microseconds
	state = STOPPED;//ACTIVE;			// we set the neuralcontroller state to active 

	Travolta->send ("motoron;");	// activates motors
	
	// Stablishes the default mode of accepting conflicting assignments 
	for (int i=0; i<NUM_MOTORS; i++)
	{
		char command[100];
		sprintf (command,"blend[cancel] %s.val,",MOTORS[i]);
		Travolta->send(command);
		usleep (1000);
	}
	
	// Makes Aibo take the starting position  //////////////////////////
		// sends broadbase position to joints
	for (int i=0; i<NUM_MOTORS; i++)
	{
		char command[100];
		sprintf (command,"%s.val=%d smooth:2800,",MOTORS[i],BROADBASE_ANGLE[i]);
		Travolta->send(command);
		usleep (10000);
	}
	usleep(2000000);
		// sends sleeping position to joints
	for (int i=0; i<NUM_MOTORS; i++)
	{
		char command2[100];
		sprintf (command2,"%s.val=%d smooth:2800,",MOTORS[i],SLEEPING_ANGLE[i]);
		Travolta->send(command2);
		usleep(10000);
	}
	usleep(2000000);
		// sends stand up position to joints
	for (int i=0; i<NUM_MOTORS; i++)
	{
		char command3[100];
		sprintf (command3,"%s.val=%d smooth:2800,",MOTORS[i],CROUCHING_ANGLE[i]);
		Travolta->send(command3);
		usleep(10000);
	}
	usleep(2000000);
	for (int i=0; i<NUM_MOTORS; i++)
	{
		char command4[100];
		sprintf (command4,"%s.val=%d smooth:2800,",MOTORS[i],STANDUP_ANGLE[i]);
		Travolta->send(command4);
		usleep(10000);
	}
	usleep(2000000);
  
	sensors = new double [NUM_SENSORS];	
}

void Neurocontrolador::calculaRedes()
{
	
	normalize_sensor_values();

	double previous_sensor_nets_outputs[NUM_SENSORS];
	for (int i=0; i<NUM_SENSORS; i++)   
		previous_sensor_nets_outputs[i] = output[i][0];  // previous output of sensor net 'i'
	
	////////////////////////////////////////////
	// Input vector for the J1 sensors   ///////
	////////////////////////////////////////////
	input[0] = sensors[0];				// real sensor value for LF_J1
	input[2] = previous_sensor_nets_outputs[1];		// output value of net of sensor RF_J1 in previous step
	input[4] = previous_sensor_nets_outputs[2];		// output value of net of sensor LH_J1 in previous step
	input[6] = previous_sensor_nets_outputs[3];		// output value of net of sensor RH_J1 in previous step
	input[8] = previous_sensor_nets_outputs[4];		// output value of net of sensor LF_J2 in previous step
	input[10] = previous_sensor_nets_outputs[5];	// output value of net of sensor RF_J2 in previous step
	input[12] = previous_sensor_nets_outputs[6];	// output value of net of sensor LH_J2 in previous step
	input[14] = previous_sensor_nets_outputs[7];	// output value of net of sensor RH_J2 in previous step
	input[16] = previous_sensor_nets_outputs[8];	// output value of net of sensor LF_J3 in previous step
	input[18] = previous_sensor_nets_outputs[9];	// output value of net of sensor RF_J3 in previous step
	input[20] = previous_sensor_nets_outputs[10];   // output value of net of sensor LH_J3 in previous step
	input[22] = previous_sensor_nets_outputs[11];   // output value of net of sensor RH_J3 in previous step
	input[1] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[3] = output[13][0];					// output value of net of actuator RF_J1 in previous step
	input[5] = output[14][0];					// output value of net of actuator LH_J1 in previous step
	input[7] = output[15][0];					// output value of net of actuator RH_J1 in previous step
	input[9] = output[16][0];					// output value of net of actuator LF_J2 in previous step
	input[11] = output[17][0];				// output value of net of actuator RF_J2 in previous step
	input[13] = output[18][0];				// output value of net of actuator LH_J2 in previous step
	input[15] = output[19][0];				// output value of net of actuator RH_J2 in previous step
	input[17] = output[20][0];				// output value of net of actuator LF_J3 in previous step
	input[19] = output[21][0];				// output value of net of actuator RF_J3 in previous step
	input[21] = output[22][0];				// output value of net of actuator LH_J3 in previous step
	input[23] = output[23][0];				// output value of net of actuator RH_J3 in previous step
	quantify_ANN_values ();
	pNet[0]->activate (input,output[0]);		// sensor LFLEG_J1

	// Input vector for the RF_J1 sensor	
	input[0] = sensors[6];				// real sensor value for RF_J1
	input[2] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[4] = previous_sensor_nets_outputs[2];		// output value of net of sensor LH_J1 in previous step
	input[6] = previous_sensor_nets_outputs[3];		// output value of net of sensor RH_J1 in previous step
	input[8] = previous_sensor_nets_outputs[4];		// output value of net of sensor LF_J2 in previous step
	input[10] = previous_sensor_nets_outputs[5];	// output value of net of sensor RF_J2 in previous step
	input[12] = previous_sensor_nets_outputs[6];	// output value of net of sensor LH_J2 in previous step
	input[14] = previous_sensor_nets_outputs[7];	// output value of net of sensor RH_J2 in previous step
	input[16] = previous_sensor_nets_outputs[8];	// output value of net of sensor LF_J3 in previous step
	input[18] = previous_sensor_nets_outputs[9];	// output value of net of sensor RF_J3 in previous step
	input[20] = previous_sensor_nets_outputs[10];   // output value of net of sensor LH_J3 in previous step
	input[22] = previous_sensor_nets_outputs[11];   // output value of net of sensor RH_J3 in previous step
	input[1] = output[13][0];					// output value of net of actuator RF_J1 in previous step
	input[3] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[5] = output[14][0];					// output value of net of actuator LH_J1 in previous step
	input[7] = output[15][0];					// output value of net of actuator RH_J1 in previous step
	input[9] = output[16][0];					// output value of net of actuator LF_J2 in previous step
	input[11] = output[17][0];				// output value of net of actuator RF_J2 in previous step
	input[13] = output[18][0];				// output value of net of actuator LH_J2 in previous step
	input[15] = output[19][0];				// output value of net of actuator RH_J2 in previous step
	input[17] = output[20][0];				// output value of net of actuator LF_J3 in previous step
	input[19] = output[21][0];				// output value of net of actuator RF_J3 in previous step
	input[21] = output[22][0];				// output value of net of actuator LH_J3 in previous step
	input[23] = output[23][0];				// output value of net of actuator RH_J3 in previous step
	quantify_ANN_values ();
	pNet[1]->activate (input,output[1]);		// sensor RFLEG_J1

	// Input vector for the LH_J1 sensor	
	input[0] = sensors[3];				// real sensor value for LH_J1
	input[2] = previous_sensor_nets_outputs[3];		// output value of net of sensor RH_J1 in previous step
	input[4] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[6] = previous_sensor_nets_outputs[1];		// output value of net of sensor RF_J1 in previous step
	input[8] = previous_sensor_nets_outputs[4];		// output value of net of sensor LF_J2 in previous step
	input[10] = previous_sensor_nets_outputs[5];	// output value of net of sensor RF_J2 in previous step
	input[12] = previous_sensor_nets_outputs[6];	// output value of net of sensor LH_J2 in previous step
	input[14] = previous_sensor_nets_outputs[7];	// output value of net of sensor RH_J2 in previous step
	input[16] = previous_sensor_nets_outputs[8];	// output value of net of sensor LF_J3 in previous step
	input[18] = previous_sensor_nets_outputs[9];	// output value of net of sensor RF_J3 in previous step
	input[20] = previous_sensor_nets_outputs[10];   // output value of net of sensor LH_J3 in previous step
	input[22] = previous_sensor_nets_outputs[11];   // output value of net of sensor RH_J3 in previous step
	input[1] = output[14][0];					// output value of net of actuator LH_J1 in previous step
	input[3] = output[15][0];					// output value of net of actuator RH_J1 in previous step
	input[5] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[7] = output[13][0];					// output value of net of actuator RF_J1 in previous step
	input[9] = output[16][0];					// output value of net of actuator LF_J2 in previous step
	input[11] = output[17][0];				// output value of net of actuator RF_J2 in previous step
	input[13] = output[18][0];				// output value of net of actuator LH_J2 in previous step
	input[15] = output[19][0];				// output value of net of actuator RH_J2 in previous step
	input[17] = output[20][0];				// output value of net of actuator LF_J3 in previous step
	input[19] = output[21][0];				// output value of net of actuator RF_J3 in previous step
	input[21] = output[22][0];				// output value of net of actuator LH_J3 in previous step
	input[23] = output[23][0];				// output value of net of actuator RH_J3 in previous step
	quantify_ANN_values ();
	pNet[2]->activate (input,output[2]);		// sensor LHLEG_J1
	
	// Input vector for the RH_J1 sensor
	input[0] = sensors[9];				// real sensor value for RH_J1
	input[2] = previous_sensor_nets_outputs[2];		// output value of net of sensor LH_J1 in previous step
	input[4] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[6] = previous_sensor_nets_outputs[1];		// output value of net of sensor RF_J1 in previous step
	input[8] = previous_sensor_nets_outputs[4];		// output value of net of sensor LF_J2 in previous step
	input[10] = previous_sensor_nets_outputs[5];	// output value of net of sensor RF_J2 in previous step
	input[12] = previous_sensor_nets_outputs[6];	// output value of net of sensor LH_J2 in previous step
	input[14] = previous_sensor_nets_outputs[7];	// output value of net of sensor RH_J2 in previous step
	input[16] = previous_sensor_nets_outputs[8];	// output value of net of sensor LF_J3 in previous step
	input[18] = previous_sensor_nets_outputs[9];	// output value of net of sensor RF_J3 in previous step
	input[20] = previous_sensor_nets_outputs[10];   // output value of net of sensor LH_J3 in previous step
	input[22] = previous_sensor_nets_outputs[11];   // output value of net of sensor RH_J3 in previous step
	input[1] = output[15][0];					// output value of net of actuator RH_J1 in previous step
	input[3] = output[14][0];					// output value of net of actuator LH_J1 in previous step
	input[5] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[7] = output[13][0];					// output value of net of actuator RF_J1 in previous step
	input[9] = output[16][0];					// output value of net of actuator LF_J2 in previous step
	input[11] = output[17][0];				// output value of net of actuator RF_J2 in previous step
	input[13] = output[18][0];				// output value of net of actuator LH_J2 in previous step
	input[15] = output[19][0];				// output value of net of actuator RH_J2 in previous step
	input[17] = output[20][0];				// output value of net of actuator LF_J3 in previous step
	input[19] = output[21][0];				// output value of net of actuator RF_J3 in previous step
	input[21] = output[22][0];				// output value of net of actuator LH_J3 in previous step
	input[23] = output[23][0];				// output value of net of actuator RH_J3 in previous step
	quantify_ANN_values ();
	pNet[3]->activate (input,output[3]);		// sensor RHLEG_J1


	////////////////////////////////////////////
	// Input vector for the J2 sensors   ///////
	////////////////////////////////////////////
	input[0] = sensors[1];				// real sensor value for LF_J2
	input[2] = previous_sensor_nets_outputs[5];		// output value of net of sensor RF_J2 in previous step
	input[4] = previous_sensor_nets_outputs[6];		// output value of net of sensor LH_J2 in previous step
	input[6] = previous_sensor_nets_outputs[7];		// output value of net of sensor RH_J2 in previous step
	input[8] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[10] = previous_sensor_nets_outputs[1];	// output value of net of sensor RF_J1 in previous step
	input[12] = previous_sensor_nets_outputs[2];	// output value of net of sensor LH_J1 in previous step
	input[14] = previous_sensor_nets_outputs[3];	// output value of net of sensor RH_J1 in previous step
	input[16] = previous_sensor_nets_outputs[8];	// output value of net of sensor LF_J3 in previous step
	input[18] = previous_sensor_nets_outputs[9];	// output value of net of sensor RF_J3 in previous step
	input[20] = previous_sensor_nets_outputs[10];   // output value of net of sensor LH_J3 in previous step
	input[22] = previous_sensor_nets_outputs[11];   // output value of net of sensor RH_J3 in previous step
	input[1] = output[16][0];					// output value of net of actuator LF_J2 in previous step
	input[3] = output[17][0];					// output value of net of actuator RF_J2 in previous step
	input[5] = output[18][0];					// output value of net of actuator LH_J2 in previous step
	input[7] = output[19][0];					// output value of net of actuator RH_J2 in previous step
	input[9] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[11] = output[13][0];				// output value of net of actuator RF_J1 in previous step
	input[13] = output[14][0];				// output value of net of actuator LH_J1 in previous step
	input[15] = output[15][0];				// output value of net of actuator RH_J1 in previous step
	input[17] = output[20][0];				// output value of net of actuator LF_J3 in previous step
	input[19] = output[21][0];				// output value of net of actuator RF_J3 in previous step
	input[21] = output[22][0];				// output value of net of actuator LH_J3 in previous step
	input[23] = output[23][0];				// output value of net of actuator RH_J3 in previous step
	quantify_ANN_values ();
	pNet[4]->activate (input,output[4]);		// sensor LFLEG_J2

	// Input vector for the RF_J2 sensor	
	input[0] = sensors[7];				// real sensor value for RF_J2
	input[2] = previous_sensor_nets_outputs[4];		// output value of net of sensor LF_J2 in previous step
	input[4] = previous_sensor_nets_outputs[6];		// output value of net of sensor LH_J2 in previous step
	input[6] = previous_sensor_nets_outputs[7];		// output value of net of sensor RH_J2 in previous step
	input[8] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[10] = previous_sensor_nets_outputs[1];	// output value of net of sensor RF_J1 in previous step
	input[12] = previous_sensor_nets_outputs[2];	// output value of net of sensor LH_J1 in previous step
	input[14] = previous_sensor_nets_outputs[3];	// output value of net of sensor RH_J1 in previous step
	input[16] = previous_sensor_nets_outputs[8];	// output value of net of sensor LF_J3 in previous step
	input[18] = previous_sensor_nets_outputs[9];	// output value of net of sensor RF_J3 in previous step
	input[20] = previous_sensor_nets_outputs[10];   // output value of net of sensor LH_J3 in previous step
	input[22] = previous_sensor_nets_outputs[11];   // output value of net of sensor RH_J3 in previous step
	input[1] = output[17][0];					// output value of net of actuator RF_J2 in previous step
	input[3] = output[16][0];					// output value of net of actuator LF_J2 in previous step
	input[5] = output[18][0];					// output value of net of actuator LH_J2 in previous step
	input[7] = output[19][0];					// output value of net of actuator RH_J2 in previous step
	input[9] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[11] = output[13][0];				// output value of net of actuator RF_J1 in previous step
	input[13] = output[14][0];				// output value of net of actuator LH_J1 in previous step
	input[15] = output[15][0];				// output value of net of actuator RH_J1 in previous step
	input[17] = output[20][0];				// output value of net of actuator LF_J3 in previous step
	input[19] = output[21][0];				// output value of net of actuator RF_J3 in previous step
	input[21] = output[22][0];				// output value of net of actuator LH_J3 in previous step
	input[23] = output[23][0];				// output value of net of actuator RH_J3 in previous step
	quantify_ANN_values ();
	pNet[5]->activate (input,output[5]);		// sensor RFLEG_J2

	// Input vector for the LH_J2 sensor	
	input[0] = sensors[4];				// real sensor value for LH_J2
	input[2] = previous_sensor_nets_outputs[7];		// output value of net of sensor RH_J2 in previous step
	input[4] = previous_sensor_nets_outputs[4];		// output value of net of sensor LF_J2 in previous step
	input[6] = previous_sensor_nets_outputs[5];		// output value of net of sensor RF_J2 in previous step
	input[8] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[10] = previous_sensor_nets_outputs[1];	// output value of net of sensor RF_J1 in previous step
	input[12] = previous_sensor_nets_outputs[2];	// output value of net of sensor LH_J1 in previous step
	input[14] = previous_sensor_nets_outputs[3];	// output value of net of sensor RH_J1 in previous step
	input[16] = previous_sensor_nets_outputs[8];	// output value of net of sensor LF_J3 in previous step
	input[18] = previous_sensor_nets_outputs[9];	// output value of net of sensor RF_J3 in previous step
	input[20] = previous_sensor_nets_outputs[10];   // output value of net of sensor LH_J3 in previous step
	input[22] = previous_sensor_nets_outputs[11];   // output value of net of sensor RH_J3 in previous step
	input[1] = output[18][0];					// output value of net of actuator LH_J2 in previous step
	input[3] = output[19][0];					// output value of net of actuator RH_J2 in previous step
	input[5] = output[16][0];					// output value of net of actuator LF_J2 in previous step
	input[7] = output[17][0];					// output value of net of actuator RF_J2 in previous step
	input[9] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[11] = output[13][0];				// output value of net of actuator RF_J1 in previous step
	input[13] = output[14][0];				// output value of net of actuator LH_J1 in previous step
	input[15] = output[15][0];				// output value of net of actuator RH_J1 in previous step
	input[17] = output[20][0];				// output value of net of actuator LF_J3 in previous step
	input[19] = output[21][0];				// output value of net of actuator RF_J3 in previous step
	input[21] = output[22][0];				// output value of net of actuator LH_J3 in previous step
	input[23] = output[23][0];				// output value of net of actuator RH_J3 in previous step
	quantify_ANN_values ();
	pNet[6]->activate (input,output[6]);		// sensor LHLEG_J2
	
	// Input vector for the RH_J2 sensor
	input[0] = sensors[10];				// real sensor value for RH_J2
	input[2] = previous_sensor_nets_outputs[6];		// output value of net of sensor LH_J2 in previous step
	input[4] = previous_sensor_nets_outputs[4];		// output value of net of sensor LF_J2 in previous step
	input[6] = previous_sensor_nets_outputs[5];		// output value of net of sensor RF_J2 in previous step
	input[8] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[10] = previous_sensor_nets_outputs[1];	// output value of net of sensor RF_J1 in previous step
	input[12] = previous_sensor_nets_outputs[2];	// output value of net of sensor LH_J1 in previous step
	input[14] = previous_sensor_nets_outputs[3];	// output value of net of sensor RH_J1 in previous step
	input[16] = previous_sensor_nets_outputs[8];	// output value of net of sensor LF_J3 in previous step
	input[18] = previous_sensor_nets_outputs[9];	// output value of net of sensor RF_J3 in previous step
	input[20] = previous_sensor_nets_outputs[10];   // output value of net of sensor LH_J3 in previous step
	input[22] = previous_sensor_nets_outputs[11];   // output value of net of sensor RH_J3 in previous step
	input[1] = output[19][0];					// output value of net of actuator RH_J2 in previous step
	input[3] = output[18][0];					// output value of net of actuator LH_J2 in previous step
	input[5] = output[16][0];					// output value of net of actuator LF_J2 in previous step
	input[7] = output[17][0];					// output value of net of actuator RF_J2 in previous step
	input[9] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[11] = output[13][0];				// output value of net of actuator RF_J1 in previous step
	input[13] = output[14][0];				// output value of net of actuator LH_J1 in previous step
	input[15] = output[15][0];				// output value of net of actuator RH_J1 in previous step
	input[17] = output[20][0];				// output value of net of actuator LF_J3 in previous step
	input[19] = output[21][0];				// output value of net of actuator RF_J3 in previous step
	input[21] = output[22][0];				// output value of net of actuator LH_J3 in previous step
	input[23] = output[23][0];				// output value of net of actuator RH_J3 in previous step
	quantify_ANN_values ();
	pNet[7]->activate (input,output[7]);		// sensor RHLEG_J2

	
	////////////////////////////////////////////
	// Input vector for the J3 sensors   ///////
	////////////////////////////////////////////
	input[0] = sensors[2];				// real sensor value for LF_J3
	input[2] = previous_sensor_nets_outputs[9];		// output value of net of sensor RF_J3 in previous step
	input[4] = previous_sensor_nets_outputs[10];		// output value of net of sensor LH_J3 in previous step
	input[6] = previous_sensor_nets_outputs[11];		// output value of net of sensor RH_J3 in previous step
	input[8] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[10] = previous_sensor_nets_outputs[1];	// output value of net of sensor RF_J1 in previous step
	input[12] = previous_sensor_nets_outputs[2];	// output value of net of sensor LH_J1 in previous step
	input[14] = previous_sensor_nets_outputs[3];	// output value of net of sensor RH_J1 in previous step
	input[16] = previous_sensor_nets_outputs[4];	// output value of net of sensor LF_J2 in previous step
	input[18] = previous_sensor_nets_outputs[5];	// output value of net of sensor RF_J2 in previous step
	input[20] = previous_sensor_nets_outputs[6];   // output value of net of sensor LH_J2 in previous step
	input[22] = previous_sensor_nets_outputs[7];   // output value of net of sensor RH_J2 in previous step
	input[1] = output[20][0];					// output value of net of actuator LF_J3 in previous step
	input[3] = output[21][0];					// output value of net of actuator RF_J3 in previous step
	input[5] = output[22][0];					// output value of net of actuator LH_J3 in previous step
	input[7] = output[23][0];					// output value of net of actuator RH_J3 in previous step
	input[9] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[11] = output[13][0];				// output value of net of actuator RF_J1 in previous step
	input[13] = output[14][0];				// output value of net of actuator LH_J1 in previous step
	input[15] = output[15][0];				// output value of net of actuator RH_J1 in previous step
	input[17] = output[16][0];				// output value of net of actuator LF_J2 in previous step
	input[19] = output[17][0];				// output value of net of actuator RF_J2 in previous step
	input[21] = output[18][0];				// output value of net of actuator LH_J2 in previous step
	input[23] = output[19][0];				// output value of net of actuator RH_J2 in previous step
	quantify_ANN_values ();
	pNet[8]->activate (input,output[8]);		// sensor LFLEG_J3

	// Input vector for the RF_J3 sensor	
	input[0] = sensors[8];				// real sensor value for RF_J3
	input[2] = previous_sensor_nets_outputs[8];		// output value of net of sensor LF_J3 in previous step
	input[4] = previous_sensor_nets_outputs[10];		// output value of net of sensor LH_J3 in previous step
	input[6] = previous_sensor_nets_outputs[11];		// output value of net of sensor RH_J3 in previous step
	input[8] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[10] = previous_sensor_nets_outputs[1];	// output value of net of sensor RF_J1 in previous step
	input[12] = previous_sensor_nets_outputs[2];	// output value of net of sensor LH_J1 in previous step
	input[14] = previous_sensor_nets_outputs[3];	// output value of net of sensor RH_J1 in previous step
	input[16] = previous_sensor_nets_outputs[4];	// output value of net of sensor LF_J2 in previous step
	input[18] = previous_sensor_nets_outputs[5];	// output value of net of sensor RF_J2 in previous step
	input[20] = previous_sensor_nets_outputs[6];   // output value of net of sensor LH_J2 in previous step
	input[22] = previous_sensor_nets_outputs[7];   // output value of net of sensor RH_J2 in previous step
	input[1] = output[21][0];					// output value of net of actuator RF_J3 in previous step
	input[3] = output[20][0];					// output value of net of actuator LF_J3 in previous step
	input[5] = output[22][0];					// output value of net of actuator LH_J3 in previous step
	input[7] = output[23][0];					// output value of net of actuator RH_J3 in previous step
	input[9] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[11] = output[13][0];				// output value of net of actuator RF_J1 in previous step
	input[13] = output[14][0];				// output value of net of actuator LH_J1 in previous step
	input[15] = output[15][0];				// output value of net of actuator RH_J1 in previous step
	input[17] = output[16][0];				// output value of net of actuator LF_J2 in previous step
	input[19] = output[17][0];				// output value of net of actuator RF_J2 in previous step
	input[21] = output[18][0];				// output value of net of actuator LH_J2 in previous step
	input[23] = output[19][0];				// output value of net of actuator RH_J2 in previous step
	quantify_ANN_values ();
	pNet[9]->activate (input,output[9]);		// sensor RFLEG_J3

	// Input vector for the LH_J3 sensor	
	input[0] = sensors[5];				// real sensor value for LH_J3
	input[2] = previous_sensor_nets_outputs[11];		// output value of net of sensor RH_J3 in previous step
	input[4] = previous_sensor_nets_outputs[8];		// output value of net of sensor LF_J3 in previous step
	input[6] = previous_sensor_nets_outputs[9];		// output value of net of sensor RF_J3 in previous step
	input[8] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[10] = previous_sensor_nets_outputs[1];	// output value of net of sensor RF_J1 in previous step
	input[12] = previous_sensor_nets_outputs[2];	// output value of net of sensor LH_J1 in previous step
	input[14] = previous_sensor_nets_outputs[3];	// output value of net of sensor RH_J1 in previous step
	input[16] = previous_sensor_nets_outputs[4];	// output value of net of sensor LF_J2 in previous step
	input[18] = previous_sensor_nets_outputs[5];	// output value of net of sensor RF_J2 in previous step
	input[20] = previous_sensor_nets_outputs[6];   // output value of net of sensor LH_J2 in previous step
	input[22] = previous_sensor_nets_outputs[7];   // output value of net of sensor RH_J2 in previous step
	input[1] = output[22][0];					// output value of net of actuator LH_J3 in previous step
	input[3] = output[23][0];					// output value of net of actuator RH_J3 in previous step
	input[5] = output[20][0];					// output value of net of actuator LF_J3 in previous step
	input[7] = output[21][0];					// output value of net of actuator RF_J3 in previous step
	input[9] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[11] = output[13][0];				// output value of net of actuator RF_J1 in previous step
	input[13] = output[14][0];				// output value of net of actuator LH_J1 in previous step
	input[15] = output[15][0];				// output value of net of actuator RH_J1 in previous step
	input[17] = output[16][0];				// output value of net of actuator LF_J2 in previous step
	input[19] = output[17][0];				// output value of net of actuator RF_J2 in previous step
	input[21] = output[18][0];				// output value of net of actuator LH_J2 in previous step
	input[23] = output[19][0];				// output value of net of actuator RH_J2 in previous step
	quantify_ANN_values ();
	pNet[10]->activate (input,output[10]);		// sensor LHLEG_J3
	
	// Input vector for the RH_J3 sensor
	input[0] = sensors[1];				// real sensor value for RH_J3
	input[2] = previous_sensor_nets_outputs[10];		// output value of net of sensor LH_J3 in previous step
	input[4] = previous_sensor_nets_outputs[8];		// output value of net of sensor LF_J3 in previous step
	input[6] = previous_sensor_nets_outputs[9];		// output value of net of sensor RF_J3 in previous step
	input[8] = previous_sensor_nets_outputs[0];		// output value of net of sensor LF_J1 in previous step
	input[10] = previous_sensor_nets_outputs[1];	// output value of net of sensor RF_J1 in previous step
	input[12] = previous_sensor_nets_outputs[2];	// output value of net of sensor LH_J1 in previous step
	input[14] = previous_sensor_nets_outputs[3];	// output value of net of sensor RH_J1 in previous step
	input[16] = previous_sensor_nets_outputs[4];	// output value of net of sensor LF_J2 in previous step
	input[18] = previous_sensor_nets_outputs[5];	// output value of net of sensor RF_J2 in previous step
	input[20] = previous_sensor_nets_outputs[6];   // output value of net of sensor LH_J2 in previous step
	input[22] = previous_sensor_nets_outputs[7];   // output value of net of sensor RH_J2 in previous step
	input[1] = output[23][0];					// output value of net of actuator RH_J3 in previous step
	input[3] = output[22][0];					// output value of net of actuator LH_J3 in previous step
	input[5] = output[20][0];					// output value of net of actuator LF_J3 in previous step
	input[7] = output[21][0];					// output value of net of actuator RF_J3 in previous step
	input[9] = output[12][0];					// output value of net of actuator LF_J1 in previous step
	input[11] = output[13][0];				// output value of net of actuator RF_J1 in previous step
	input[13] = output[14][0];				// output value of net of actuator LH_J1 in previous step
	input[15] = output[15][0];				// output value of net of actuator RH_J1 in previous step
	input[17] = output[16][0];				// output value of net of actuator LF_J2 in previous step
	input[19] = output[17][0];				// output value of net of actuator RF_J2 in previous step
	input[21] = output[18][0];				// output value of net of actuator LH_J2 in previous step
	input[23] = output[19][0];				// output value of net of actuator RH_J2 in previous step
	quantify_ANN_values ();
	pNet[11]->activate (input,output[11]);		// sensor RHLEG_J3

// Calculo de las nuevas salidas de las redes neuronales de las joints	
	double previous_actuators_nets_outputs[NUM_JOINTS];
	for (int i=0; i< NUM_JOINTS; i++)
		previous_actuators_nets_outputs[i] = output[i+12][0];	
	
					///////////////////////////////////
					// Colculus for J1 joints /////////
		/////////////////////////////////////////////////////////////
					
	// Input vector for LF_J1 actuator
	input[0] = output[0][0];							// sensor LF_J1
	input[1] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[2] = output[1][0];							// sensor RF_J1
	input[3] = previous_actuators_nets_outputs[1];  // actuator RF_J1
	input[4] = output[2][0];							// sensor LH_J1
	input[5] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[6] = output[3][0];							// sensor RH_J1
	input[7] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[8] = output[4][0];							// sensor LF_J2
	input[9] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[10] = output[5][0];							// sensor RF_J2
	input[11] = previous_actuators_nets_outputs[5];  // actuator RF_J2
	input[12] = output[6][0];							// sensor LH_J2
	input[13] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[14] = output[7][0];							// sensor RH_J2
	input[15] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	input[16] = output[8][0];							// sensor LF_J3
	input[17] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[18] = output[9][0];							// sensor RF_J3
	input[19] = previous_actuators_nets_outputs[9];  // actuator RF_J3
	input[20] = output[10][0];							// sensor LH_J3
	input[21] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[22] = output[11][0];							// sensor RH_J3
	input[23] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	quantify_ANN_values ();
	pNet[12]->activate (input,output[12]);	// actuador LFLEG_J1
	
	// Input vector for RF_J1 actuator
	input[0] = output[1][0];							// sensor RF_J1
	input[1] = previous_actuators_nets_outputs[1];  // actuator RF_J1
	input[2] = output[0][0];							// sensor LF_J1
	input[3] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[4] = output[2][0];							// sensor LH_J1
	input[5] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[6] = output[3][0];							// sensor RH_J1
	input[7] = previous_actuators_nets_outputs[3];  // actuator RH_J1

	input[8] = output[4][0];							// sensor LF_J2
	input[9] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[10] = output[5][0];							// sensor RF_J2
	input[11] = previous_actuators_nets_outputs[5];  // actuator RF_J2
	input[12] = output[6][0];							// sensor LH_J2
	input[13] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[14] = output[7][0];							// sensor RH_J2
	input[15] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	input[16] = output[8][0];							// sensor LF_J3
	input[17] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[18] = output[9][0];							// sensor RF_J3
	input[19] = previous_actuators_nets_outputs[9];  // actuator RF_J3
	input[20] = output[10][0];							// sensor LH_J3
	input[21] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[22] = output[11][0];							// sensor RH_J3
	input[23] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	quantify_ANN_values ();
	pNet[13]->activate (input,output[13]);	// actuador RFLEG_J1
	
	// Input vector for LH_J1 actuator
	input[0] = output[2][0];							// sensor LH_J1
	input[1] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[2] = output[3][0];							// sensor RH_J1
	input[3] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[4] = output[0][0];							// sensor LF_J1
	input[5] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[6] = output[1][0];							// sensor RF_J1
	input[7] = previous_actuators_nets_outputs[1];  // actuator RF_J1

	input[8] = output[4][0];							// sensor LF_J2
	input[9] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[10] = output[5][0];							// sensor RF_J2
	input[11] = previous_actuators_nets_outputs[5];  // actuator RF_J2
	input[12] = output[6][0];							// sensor LH_J2
	input[13] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[14] = output[7][0];							// sensor RH_J2
	input[15] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	input[16] = output[8][0];							// sensor LF_J3
	input[17] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[18] = output[9][0];							// sensor RF_J3
	input[19] = previous_actuators_nets_outputs[9];  // actuator RF_J3
	input[20] = output[10][0];							// sensor LH_J3
	input[21] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[22] = output[11][0];							// sensor RH_J3
	input[23] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	quantify_ANN_values ();
	pNet[14]->activate (input,output[14]);	// actuador LHLEG_J1
	
	// Input vector for RH_J1 actuator
	input[0] = output[3][0];							// sensor RH_J1
	input[1] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[2] = output[2][0];							// sensor LH_J1
	input[3] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[4] = output[0][0];							// sensor LF_J1
	input[5] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[6] = output[1][0];							// sensor RF_J1
	input[7] = previous_actuators_nets_outputs[1];  // actuator RF_J1

	input[8] = output[4][0];							// sensor LF_J2
	input[9] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[10] = output[5][0];							// sensor RF_J2
	input[11] = previous_actuators_nets_outputs[5];  // actuator RF_J2
	input[12] = output[6][0];							// sensor LH_J2
	input[13] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[14] = output[7][0];							// sensor RH_J2
	input[15] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	input[16] = output[8][0];							// sensor LF_J3
	input[17] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[18] = output[9][0];							// sensor RF_J3
	input[19] = previous_actuators_nets_outputs[9];  // actuator RF_J3
	input[20] = output[10][0];							// sensor LH_J3
	input[21] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[22] = output[11][0];							// sensor RH_J3
	input[23] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	quantify_ANN_values ();
	pNet[15]->activate (input,output[15]);	// actuador RHLEG_J1
	
	
					///////////////////////////////////
					// Colculus for J2 joints /////////
		/////////////////////////////////////////////////////////////
					
	// Input vector for LF_J2 actuator
	input[0] = output[4][0];							// sensor LF_J2
	input[1] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[2] = output[5][0];							// sensor RF_J2
	input[3] = previous_actuators_nets_outputs[5];  // actuator RF_J2
	input[4] = output[6][0];							// sensor LH_J2
	input[5] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[6] = output[7][0];							// sensor RH_J2
	input[7] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	
	input[8] = output[0][0];							// sensor LF_J1
	input[9] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[10] = output[1][0];							// sensor RF_J1
	input[11] = previous_actuators_nets_outputs[1];  // actuator RF_J1
	input[12] = output[2][0];							// sensor LH_J1
	input[13] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[14] = output[3][0];							// sensor RH_J1
	input[15] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[16] = output[8][0];							// sensor LF_J3
	input[17] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[18] = output[9][0];							// sensor RF_J3
	input[19] = previous_actuators_nets_outputs[9];  // actuator RF_J3
	input[20] = output[10][0];							// sensor LH_J3
	input[21] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[22] = output[11][0];							// sensor RH_J3
	input[23] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	quantify_ANN_values ();
	pNet[16]->activate (input,output[16]);	// actuador LFLEG_J2
	
	// Input vector for RF_J2 actuator
	input[0] = output[5][0];							// sensor RF_J2
	input[1] = previous_actuators_nets_outputs[5];  // actuator RF_J2
	input[2] = output[4][0];							// sensor LF_J2
	input[3] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[4] = output[6][0];							// sensor LH_J2
	input[5] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[6] = output[7][0];							// sensor RH_J2
	input[7] = previous_actuators_nets_outputs[7];  // actuator RH_J2

	input[8] = output[0][0];							// sensor LF_J1
	input[9] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[10] = output[1][0];							// sensor RF_J1
	input[11] = previous_actuators_nets_outputs[1];  // actuator RF_J1
	input[12] = output[2][0];							// sensor LH_J1
	input[13] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[14] = output[3][0];							// sensor RH_J1
	input[15] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[16] = output[8][0];							// sensor LF_J3
	input[17] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[18] = output[9][0];							// sensor RF_J3
	input[19] = previous_actuators_nets_outputs[9];  // actuator RF_J3
	input[20] = output[10][0];							// sensor LH_J3
	input[21] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[22] = output[11][0];							// sensor RH_J3
	input[23] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	quantify_ANN_values ();
	pNet[17]->activate (input,output[17]);	// actuador RFLEG_J2
	
	// Input vector for LH_J2 actuator
	input[0] = output[6][0];							// sensor LH_J2
	input[1] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[2] = output[7][0];							// sensor RH_J2
	input[3] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	input[4] = output[4][0];							// sensor LF_J2
	input[5] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[6] = output[5][0];							// sensor RF_J2
	input[7] = previous_actuators_nets_outputs[5];  // actuator RF_J2

	input[8] = output[0][0];							// sensor LF_J1
	input[9] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[10] = output[1][0];							// sensor RF_J1
	input[11] = previous_actuators_nets_outputs[1];  // actuator RF_J1
	input[12] = output[2][0];							// sensor LH_J1
	input[13] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[14] = output[3][0];							// sensor RH_J1
	input[15] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[16] = output[8][0];							// sensor LF_J3
	input[17] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[18] = output[9][0];							// sensor RF_J3
	input[19] = previous_actuators_nets_outputs[9];  // actuator RF_J3
	input[20] = output[10][0];							// sensor LH_J3
	input[21] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[22] = output[11][0];							// sensor RH_J3
	input[23] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	quantify_ANN_values ();
	pNet[18]->activate (input,output[18]);	// actuador LHLEG_J2
	
	// Input vector for RH_J2 actuator
	input[0] = output[7][0];							// sensor RH_J2
	input[1] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	input[2] = output[6][0];							// sensor LH_J2
	input[3] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[4] = output[4][0];							// sensor LF_J2
	input[5] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[6] = output[5][0];							// sensor RF_J2
	input[7] = previous_actuators_nets_outputs[5];  // actuator RF_J2

	input[8] = output[0][0];							// sensor LF_J1
	input[9] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[10] = output[1][0];							// sensor RF_J1
	input[11] = previous_actuators_nets_outputs[1];  // actuator RF_J1
	input[12] = output[2][0];							// sensor LH_J1
	input[13] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[14] = output[3][0];							// sensor RH_J1
	input[15] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[16] = output[8][0];							// sensor LF_J3
	input[17] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[18] = output[9][0];							// sensor RF_J3
	input[19] = previous_actuators_nets_outputs[9];  // actuator RF_J3
	input[20] = output[10][0];							// sensor LH_J3
	input[21] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[22] = output[11][0];							// sensor RH_J3
	input[23] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	quantify_ANN_values ();
	pNet[19]->activate (input,output[19]);	// actuador RHLEG_J2


					///////////////////////////////////
					// Colculus for J3 joints /////////
		/////////////////////////////////////////////////////////////
					
	// Input vector for LF_J3 actuator
	input[0] = output[8][0];							// sensor LF_J3
	input[1] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[2] = output[9][0];							// sensor RF_J3
	input[3] = previous_actuators_nets_outputs[9];  // actuator RF_J3
	input[4] = output[10][0];							// sensor LH_J3
	input[5] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[6] = output[11][0];							// sensor RH_J3
	input[7] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	
	input[8] = output[0][0];							// sensor LF_J1
	input[9] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[10] = output[1][0];							// sensor RF_J1
	input[11] = previous_actuators_nets_outputs[1];  // actuator RF_J1
	input[12] = output[2][0];							// sensor LH_J1
	input[13] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[14] = output[3][0];							// sensor RH_J1
	input[15] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[16] = output[4][0];							// sensor LF_J2
	input[17] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[18] = output[5][0];							// sensor RF_J2
	input[19] = previous_actuators_nets_outputs[5];  // actuator RF_J2
	input[20] = output[6][0];							// sensor LH_J2
	input[21] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[22] = output[7][0];							// sensor RH_J2
	input[23] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	quantify_ANN_values ();
	pNet[20]->activate (input,output[20]);	// actuador LFLEG_J3
	
	// Input vector for RF_J3 actuator
	input[0] = output[9][0];							// sensor RF_J3
	input[1] = previous_actuators_nets_outputs[9];  // actuator RF_J3
	input[2] = output[8][0];							// sensor LF_J3
	input[3] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[4] = output[10][0];							// sensor LH_J3
	input[5] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[6] = output[11][0];							// sensor RH_J3
	input[7] = previous_actuators_nets_outputs[11];  // actuator RH_J3

	input[8] = output[0][0];							// sensor LF_J1
	input[9] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[10] = output[1][0];							// sensor RF_J1
	input[11] = previous_actuators_nets_outputs[1];  // actuator RF_J1
	input[12] = output[2][0];							// sensor LH_J1
	input[13] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[14] = output[3][0];							// sensor RH_J1
	input[15] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[16] = output[4][0];							// sensor LF_J2
	input[17] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[18] = output[5][0];							// sensor RF_J2
	input[19] = previous_actuators_nets_outputs[5];  // actuator RF_J2
	input[20] = output[6][0];							// sensor LH_J2
	input[21] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[22] = output[7][0];							// sensor RH_J2
	input[23] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	quantify_ANN_values ();
	pNet[21]->activate (input,output[21]);	// actuador RFLEG_J3
	
	// Input vector for LH_J3 actuator
	input[0] = output[10][0];							// sensor LH_J3
	input[1] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[2] = output[11][0];							// sensor RH_J3
	input[3] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	input[4] = output[8][0];							// sensor LF_J3
	input[5] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[6] = output[9][0];							// sensor RF_J3
	input[7] = previous_actuators_nets_outputs[9];  // actuator RF_J3

	input[8] = output[0][0];							// sensor LF_J1
	input[9] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[10] = output[1][0];							// sensor RF_J1
	input[11] = previous_actuators_nets_outputs[1];  // actuator RF_J1
	input[12] = output[2][0];							// sensor LH_J1
	input[13] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[14] = output[3][0];							// sensor RH_J1
	input[15] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[16] = output[4][0];							// sensor LF_J2
	input[17] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[18] = output[5][0];							// sensor RF_J2
	input[19] = previous_actuators_nets_outputs[5];  // actuator RF_J2
	input[20] = output[6][0];							// sensor LH_J2
	input[21] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[22] = output[7][0];							// sensor RH_J2
	input[23] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	quantify_ANN_values ();
	pNet[22]->activate (input,output[22]);	// actuador LHLEG_J3
	
	// Input vector for RH_J3 actuator
	input[0] = output[11][0];							// sensor RH_J3
	input[1] = previous_actuators_nets_outputs[11];  // actuator RH_J3
	input[2] = output[10][0];							// sensor LH_J3
	input[3] = previous_actuators_nets_outputs[10];  // actuator LH_J3
	input[4] = output[8][0];							// sensor LF_J3
	input[5] = previous_actuators_nets_outputs[8];  // actuator LF_J3
	input[6] = output[9][0];							// sensor RF_J3
	input[7] = previous_actuators_nets_outputs[9];  // actuator RF_J3

	input[8] = output[0][0];							// sensor LF_J1
	input[9] = previous_actuators_nets_outputs[0];  // actuator LF_J1
	input[10] = output[1][0];							// sensor RF_J1
	input[11] = previous_actuators_nets_outputs[1];  // actuator RF_J1
	input[12] = output[2][0];							// sensor LH_J1
	input[13] = previous_actuators_nets_outputs[2];  // actuator LH_J1
	input[14] = output[3][0];							// sensor RH_J1
	input[15] = previous_actuators_nets_outputs[3];  // actuator RH_J1
	input[16] = output[4][0];							// sensor LF_J2
	input[17] = previous_actuators_nets_outputs[4];  // actuator LF_J2
	input[18] = output[5][0];							// sensor RF_J2
	input[19] = previous_actuators_nets_outputs[5];  // actuator RF_J2
	input[20] = output[6][0];							// sensor LH_J2
	input[21] = previous_actuators_nets_outputs[6];  // actuator LH_J2
	input[22] = output[7][0];							// sensor RH_J2
	input[23] = previous_actuators_nets_outputs[7];  // actuator RH_J2
	quantify_ANN_values ();
	pNet[23]->activate (input,output[23]);	// actuador RHLEG_J3
		
	return;	
	
}

// to normalize means to change the range of the sensor output from its normal
// range to the ANN range (0,1) 	
void Neurocontrolador::normalize_sensor_values ()
{
	// the following normalizations are based on the max. and min values
	// that the joint sensors can give
	for (int n=0; n<3; n++)
	{
		sensors[n*3] = sensors[n*3]*3.1416/180;
		sensors[n*3] = (sensors[n*3]+2.00713333)/(2.007133333+2.68933333); // for joint J1
		sensors[n*3+1] = sensors[n*+1]*3.1416/180;
		sensors[n*3 + 1] = (sensors[n*3+1]+0.175)/(0.175+1.536);	// for joint J2
		sensors[n*3+2] = sensors[n*3+2]*3.1416/180;
		sensors[n*3 + 2] = (sensors[n*3+2]+0.437)/(0.437+2.13);	// for joint J3
	}	
	
	return;
}

void Neurocontrolador::quantify_ANN_values ()
{
	int integer;
	int precision = 1000;
	for (int n=0; n<inputDimension; n++)
	{
		//printf ("cantidad original %f ",input[n]);
		input[n] = precision*input[n];
		//printf ("cantidad por diez %f ",input[n]);
		integer = (int) input[n];
		//printf ("cantidad truncada %d ",integer);
		input[n] = (double) integer ;
		input[n] = input[n]/precision;
		//printf ("cantidad final %f\n",input[n]);
	}
	
	return;
}



void Neurocontrolador::send_commands_to_joints ()
{

	double * velocity=new double[12];  // will contain the velocities of the joints
	double  direction[12];				// will contain the direction of the movement

	// velocities of the joints. Obtained from the output of the neurons
	for (int i=0; i< NUM_JOINTS;i++)
		velocity[i] = fabs(output[i+12][0] - 0.5)*2*2.81; 

	// the direction of the velocity is also obtained from the output neuron
	// ---------------------------------------------------------------------------	
	if (output[12][0]>0.5)				// actuator LFLEG_J1
		direction[0]=0.39;		// in rads
	else									
		direction[0]=-0.58;
// ---------------------------------------------------------------------------	
	if (output[13][0]>0.5)				// actuator RFLEG_J1
		direction[1]=0.39; 
	else									
		direction[1]=-0.58;
// ---------------------------------------------------------------------------	
	if (output[14][0]>0.5)				// actuator LHLEG_J1
		direction[2]=0.0; 
	else									
		direction[2]=-0.78;
// ---------------------------------------------------------------------------	
	if (output[15][0]>0.5)				// actuator RHLEG_J1
		direction[3]=0.0; 
	else									
		direction[3]=-0.78;


	// ---------------------------------------------------------------------------	
	if (output[16][0]>0.5)				// actuator LFLEG_J2
		direction[4]=0.37; 
	else									
		direction[4]=-0.21;
// ---------------------------------------------------------------------------	
	if (output[17][0]>0.5)				// actuator RFLEG_J2
		direction[5]=0.37; 
	else									
		direction[5]=-0.21;
// ---------------------------------------------------------------------------	
	if (output[18][0]>0.5)				// actuator LHLEG_J2
		direction[6]=0.42; 
	else									 
		direction[6]=-0.21;
// ---------------------------------------------------------------------------	
	if (output[19][0]>0.5)				// actuator RHLEG_J2
		direction[7]=0.42; 
	else									 
		direction[7]=-0.21;
	
	
	// ---------------------------------------------------------------------------	
	if (output[20][0]>0.5)				// actuator LFLEG_J3
		direction[8]=1.17; 
	else									
		direction[8]=0.14;
// ---------------------------------------------------------------------------	
	if (output[21][0]>0.5)				// actuator RFLEG_J3
		direction[9]=1.17; 
	else									
		direction[9]=0.14;
// ---------------------------------------------------------------------------	
	if (output[22][0]>0.5)				// actuator LHLEG_J3
		direction[10]=1.65; 
	else									
		direction[10]=0.99;
// ---------------------------------------------------------------------------	
	if (output[23][0]>0.5)				// actuator RHLEG_J3
		direction[11]=1.65;
	else									
		direction[11]=0.99;

	for (int i=0; i<NUM_JOINTS; i++)
	{
		char command[100];
		sprintf (command,"%s.val=%f speed:%f,",JOINT_MOTORS[i],direction[i]*180/3.1416,velocity[i]*180/3.1416);
		Travolta->send(command);
		usleep(100);
	}	
	usleep(200);

	return;
}


void Neurocontrolador::readSensors()
{
	double sensorValue;
	
	for (int i=0; i<NUM_SENSORS; i++)
 	{
		Travolta->syncGetDevice(JOINT_MOTORS[i],sensorValue);		
		sensors[i] = sensorValue;
		//cout << sensors[i]<<" ";
		usleep (7000);
	}
	//cout <<endl<<flush;
	return;
}

UCallbackAction onHeadSensor(const UMessage &msg) 
{
	//cout << "Head value: "<<msg.doubleValue<<endl;
	if (msg.doubleValue > 18.00)
	{
		headCounter ++;
		if (headCounter>=10)
		{
			headCounter = 0;
			if (state == ACTIVE)
			{
				state = STOPPED;
				cout << "NEUROCONTROLLER STOPPED. PRESS AIBO'S HEAD SENSOR TO CONTINUE\n";
			}
			else
			{
				state = ACTIVE;
				cout << "NEUROCONTROLLER ACTIVATED. PRESS AIBO'S HEAD SENSOR TO STOP\n";
			}
		}
	}
	return URBI_CONTINUE;
}
UCallbackAction onbackSensorF(const UMessage &msg) 
{
	
	return URBI_CONTINUE;
}
UCallbackAction onbackSensorM(const UMessage &msg) 
{
	//	cout << "Back value: "<<msg.doubleValue<<endl;
	if (msg.doubleValue > 18.00)
	{
		backCounter++;
		if (backCounter >= 10)
		{
			backCounter = 0;
			if (blending_mode < 5)
				blending_mode++;
			else
				blending_mode = 0;
		}
	}
	return URBI_CONTINUE;
}
UCallbackAction onbackSensorR(const UMessage &msg) 
{
	
	return URBI_CONTINUE;
}


int main()
{
	int bd_mode = blending_mode;
	// creates the neural controller
	Neurocontrolador neurona ; 
	ofstream sensor_values_file;  // file to store joint values during evaluation
	
	// sets callbacks for the head and back sensors
	neurona.Travolta->send("looptag: loop {");
	neurona.Travolta->setCallback(onHeadSensor,"headSensor");
	neurona.Travolta->send("headSensor: headSensor.val&");
	neurona.Travolta->setCallback(onbackSensorF,"backSensorF");
	neurona.Travolta->send("backSensorF: backSensorF.val&");
	neurona.Travolta->setCallback(onbackSensorM,"backSensorM");
	neurona.Travolta->send("backSensorM: backSensorM.val&");
	neurona.Travolta->setCallback(onbackSensorR,"backSensorR");
	neurona.Travolta->send("backSensorR: backSensorR.val},");
	
	sensor_values_file.open ("motor_sensors_values_sync.csv");   // will contain joint values sensed
	for (int i=0;i<NUM_JOINTS;i++)
		sensor_values_file << JOINT_MOTORS[i] << " ";
	sensor_values_file << endl;
	
	neurona.Travolta->send("group faceLeds {ledF1,ledF2,ledF3,ledF4,ledF5,ledF6,ledF7,ledF8,ledF9,ledF10,ledF11,ledF12};");
	
	cout << "PRESS AIBO'S HEAD SENSOR TO START\n"<<flush;
	
	// neural control loop
	for(;;)
	{
		if (state == ACTIVE)
		{
		
			neurona.readSensors();  // read sensor values
			// stores values on a file for future analysis
			for (int i=0; i<NUM_SENSORS; i++)
				sensor_values_file  << neurona.sensors[i] << " ";
			sensor_values_file  << endl;

			neurona.calculaRedes(); // uses sensor values to activate the neural nets
		 	neurona.send_commands_to_joints();  // uses neural nets answers to activate motors
			
			if (bd_mode!=blending_mode)		// checks if blending mode has been changed
			{
				bd_mode = blending_mode;
				for (int i=0; i<NUM_MOTORS; i++)
				{
					char command[100];
					sprintf (command,"blend[%s] %s.val,",BLENDING_MODE[blending_mode],MOTORS[i]);
					neurona.Travolta->send(command);
					usleep (1000);
				}
				cout << "CHANGING BLENDING MODE TO "<<BLENDING_MODE[blending_mode]<<endl;
			}
		}
		neurona.Travolta->send("faceLeds = random(2),");
		usleep(12000);
	}
	
	sensor_values_file.close();
	
	urbi::execute();
	
	return 0;
}

void load_weights_from_file ()
{
	for (int i=0; i< 24; i++)
	{
		ifstream netfile(FILE_LOCATOR[i]);
		if (!netfile) 
		{
	    	printf ( "Urbi Controller:: Error - cannot open '%s'\n",FILE_LOCATOR[i]);
    		exit(1);
		}
  		else
		{
			printf ( "Urbi Controller:: reading network file %s\n",FILE_LOCATOR[i]);
			int type, numInputs, numUnits, numOutputs;
			// reads the heather
    		netfile >> type;
    		netfile >> numInputs;
    		netfile >> numUnits;
    		netfile >> numOutputs;
			// creates the new net
			network = new FirstOrderDynamicCTRNN(inputDimension,hiddenDimension,outputDimension,0.096); // creates the network object
			network->create(); // creates all the neurons of the network
			// reads the weight values and populates with them the new network
			double temp_Weight;
			for(int i=0;i<hiddenDimension;++i) // for each neuron of the hidden layer
			{
				neuron = network->getNeuron(i);
    			for(int j=0;j<inputDimension+hiddenDimension+outputDimension+2;++j) 
				{
					netfile >> temp_Weight;  	
					neuron->setWeight (j,temp_Weight); // sets each of its weights
				}
			}
			netfile.close();
			pNet.push_back (network);
		}
	}
}
